@@ -22,16 +22,16 @@
 //!
 //! // Signing
 //! let signing_key = SigningKey::random(&mut OsRng); // Serialize with `::to_bytes()`
-//! let verify_key = signing_key.verifying_key();
+//! let verifying_key = signing_key.verifying_key();
 //! let message = b"ECDSA proves knowledge of a secret number in the context of a single message";
 //!
 //! // Note: the signature type must be annotated or otherwise inferrable as
 //! // `Signer` has many impls of the `Signer` trait (for both regular and
 //! // recoverable signature types).
 //! let signature: recoverable::Signature = signing_key.sign(message);
-//! let recovered_key = signature.recover_verify_key(message).expect("couldn't recover pubkey");
+//! let recovered_key = signature.recover_verifying_key(message).expect("couldn't recover pubkey");
 //!
-//! assert_eq!(&verify_key, &recovered_key);
+//! assert_eq!(&verifying_key, &recovered_key);
 //! # }
 //! ```
 
@@ -42,7 +42,10 @@ use elliptic_curve::subtle::Choice;
 #[cfg(feature = "ecdsa")]
 use crate::{
     ecdsa::{
-        signature::{digest::Digest, DigestVerifier},
+        signature::{
+            digest::{Digest, FixedOutput},
+            DigestVerifier,
+        },
         VerifyingKey,
     },
     elliptic_curve::{
@@ -69,7 +72,7 @@ pub const SIZE: usize = 65;
 /// - `r`: 32-byte integer, big endian
 /// - `s`: 32-byte integer, big endian
 /// - `v`: 1-byte recovery [`Id`]
-#[derive(Copy, Clone)]
+#[derive(Copy, Clone, Eq, PartialEq)]
 pub struct Signature {
     bytes: [u8; SIZE],
 }
@@ -107,7 +110,7 @@ impl Signature {
         msg: &[u8],
         signature: &super::Signature,
     ) -> Result<Self> {
-        Self::from_digest_trial_recovery(public_key, Keccak256::new().chain(msg), signature)
+        Self::from_digest_trial_recovery(public_key, Keccak256::new_with_prefix(msg), signature)
     }
 
     /// Given a public key, message digest, and signature, use trial recovery
@@ -121,14 +124,14 @@ impl Signature {
         signature: &super::Signature,
     ) -> Result<Self>
     where
-        D: Clone + Digest<OutputSize = U32>,
+        D: Clone + Digest + FixedOutput<OutputSize = U32>,
     {
         let signature = signature.normalize_s().unwrap_or(*signature);
 
         for recovery_id in 0..=1 {
             if let Ok(recoverable_signature) = Signature::new(&signature, Id(recovery_id)) {
                 if let Ok(recovered_key) =
-                    recoverable_signature.recover_verify_key_from_digest(digest.clone())
+                    recoverable_signature.recover_verifying_key_from_digest(digest.clone())
                 {
                     if public_key == &recovered_key
                         && public_key.verify_digest(digest.clone(), &signature).is_ok()
@@ -147,19 +150,19 @@ impl Signature {
     #[cfg(all(feature = "ecdsa", feature = "keccak256"))]
     #[cfg_attr(docsrs, doc(cfg(feature = "ecdsa")))]
     #[cfg_attr(docsrs, doc(cfg(feature = "keccak256")))]
-    pub fn recover_verify_key(&self, msg: &[u8]) -> Result<VerifyingKey> {
-        self.recover_verify_key_from_digest(Keccak256::new().chain(msg))
+    pub fn recover_verifying_key(&self, msg: &[u8]) -> Result<VerifyingKey> {
+        self.recover_verifying_key_from_digest(Keccak256::new_with_prefix(msg))
     }
 
     /// Recover the public key used to create the given signature as a
     /// [`VerifyingKey`] from the provided precomputed [`Digest`].
     #[cfg(feature = "ecdsa")]
     #[cfg_attr(docsrs, doc(cfg(feature = "ecdsa")))]
-    pub fn recover_verify_key_from_digest<D>(&self, msg_digest: D) -> Result<VerifyingKey>
+    pub fn recover_verifying_key_from_digest<D>(&self, msg_digest: D) -> Result<VerifyingKey>
     where
         D: Digest<OutputSize = U32>,
     {
-        self.recover_verify_key_from_digest_bytes(&msg_digest.finalize())
+        self.recover_verifying_key_from_digest_bytes(&msg_digest.finalize())
     }
 
     /// Recover the public key used to create the given signature as a
@@ -167,7 +170,7 @@ impl Signature {
     #[cfg(feature = "ecdsa")]
     #[cfg_attr(docsrs, doc(cfg(feature = "ecdsa")))]
     #[allow(non_snake_case, clippy::many_single_char_names)]
-    pub fn recover_verify_key_from_digest_bytes(
+    pub fn recover_verifying_key_from_digest_bytes(
         &self,
         digest_bytes: &FieldBytes,
     ) -> Result<VerifyingKey> {
@@ -181,13 +184,13 @@ impl Signature {
         }
 
         let R = ProjectivePoint::from(R.unwrap());
-        let r_inv = r.invert().unwrap();
+        let r_inv = *r.invert();
         let u1 = -(r_inv * z);
         let u2 = r_inv * *s;
-        let pk = ProjectivePoint::lincomb(&ProjectivePoint::GENERATOR, &u1, &R, &u2).to_affine();
+        let pk = ProjectivePoint::lincomb(&ProjectivePoint::GENERATOR, &u1, &R, &u2);
 
         // TODO(tarcieri): ensure the signature verifies?
-        Ok(VerifyingKey::from(&pk))
+        VerifyingKey::try_from(pk)
     }
 
     /// Parse the `r` component of this signature to a [`NonZeroScalar`]
@@ -225,16 +228,6 @@ impl Debug for Signature {
     }
 }
 
-// TODO(tarcieri): derive `Eq` after const generics are available
-impl Eq for Signature {}
-
-// TODO(tarcieri): derive `PartialEq` after const generics are available
-impl PartialEq for Signature {
-    fn eq(&self, other: &Self) -> bool {
-        self.as_ref().eq(other.as_ref())
-    }
-}
-
 impl TryFrom<&[u8]> for Signature {
     type Error = Error;
 
@@ -362,8 +355,8 @@ mod tests {
     fn public_key_recovery() {
         for vector in VECTORS {
             let sig = Signature::try_from(&vector.sig[..]).unwrap();
-            let prehash = Sha256::new().chain(vector.msg);
-            let pk = sig.recover_verify_key_from_digest(prehash).unwrap();
+            let prehash = Sha256::new_with_prefix(vector.msg);
+            let pk = sig.recover_verifying_key_from_digest(prehash).unwrap();
             assert_eq!(&vector.pk[..], EncodedPoint::from(&pk).as_bytes());
         }
     }
