@@ -7,9 +7,14 @@ use core::{
     fmt::{self, Debug},
 };
 use ecdsa_core::{
-    hazmat::{rfc6979_generate_k, SignPrimitive},
+    hazmat::SignPrimitive,
     signature::{
-        digest::{BlockInput, FixedOutput, Reset, Update},
+        digest::{
+            block_buffer::Eager,
+            core_api::{BlockSizeUser, BufferKindUser, CoreProxy, FixedOutputCore},
+            generic_array::typenum::{self, IsLess, Le, NonZero},
+            Digest, FixedOutput, HashMarker, OutputSizeUser,
+        },
         DigestSigner, RandomizedDigestSigner,
     },
 };
@@ -18,13 +23,13 @@ use elliptic_curve::{
     consts::U32,
     ops::{Invert, Reduce},
     rand_core::{CryptoRng, RngCore},
-    subtle::{Choice, ConstantTimeEq},
+    subtle::{Choice, ConstantTimeEq, CtOption},
     zeroize::Zeroize,
     IsHigh,
 };
 
 #[cfg(any(feature = "keccak256", feature = "sha256"))]
-use ecdsa_core::signature::{self, digest::Digest, PrehashSignature, RandomizedSigner};
+use ecdsa_core::signature::{self, PrehashSignature, RandomizedSigner};
 
 #[cfg(feature = "pkcs8")]
 use crate::pkcs8::{self, DecodePrivateKey};
@@ -77,7 +82,7 @@ where
     Self: DigestSigner<S::Digest, S>,
 {
     fn try_sign(&self, msg: &[u8]) -> Result<S, Error> {
-        self.try_sign_digest(Digest::chain(S::Digest::new(), msg))
+        self.try_sign_digest(S::Digest::new_with_prefix(msg))
     }
 }
 
@@ -88,13 +93,22 @@ where
     Self: RandomizedDigestSigner<S::Digest, S>,
 {
     fn try_sign_with_rng(&self, rng: impl CryptoRng + RngCore, msg: &[u8]) -> Result<S, Error> {
-        self.try_sign_digest_with_rng(rng, S::Digest::new().chain(msg))
+        self.try_sign_digest_with_rng(rng, S::Digest::new_with_prefix(msg))
     }
 }
 
 impl<D> DigestSigner<D, Signature> for SigningKey
 where
-    D: BlockInput + FixedOutput<OutputSize = U32> + Clone + Default + Reset + Update,
+    D: CoreProxy + Digest + FixedOutput<OutputSize = U32>,
+    D::Core: BlockSizeUser
+        + BufferKindUser<BufferKind = Eager>
+        + Clone
+        + Default
+        + FixedOutputCore
+        + HashMarker
+        + OutputSizeUser<OutputSize = D::OutputSize>,
+    <D::Core as BlockSizeUser>::BlockSize: IsLess<typenum::U256>,
+    Le<<D::Core as BlockSizeUser>::BlockSize, typenum::U256>: NonZero,
 {
     fn try_sign_digest(&self, digest: D) -> Result<Signature, Error> {
         let sig: recoverable::Signature = self.try_sign_digest(digest)?;
@@ -104,12 +118,20 @@ where
 
 impl<D> DigestSigner<D, recoverable::Signature> for SigningKey
 where
-    D: BlockInput + FixedOutput<OutputSize = U32> + Clone + Default + Reset + Update,
+    D: CoreProxy + Digest + FixedOutput<OutputSize = U32>,
+    D::Core: BlockSizeUser
+        + BufferKindUser<BufferKind = Eager>
+        + Clone
+        + Default
+        + FixedOutputCore
+        + HashMarker
+        + OutputSizeUser<OutputSize = D::OutputSize>,
+    <D::Core as BlockSizeUser>::BlockSize: IsLess<typenum::U256>,
+    Le<<D::Core as BlockSizeUser>::BlockSize, typenum::U256>: NonZero,
 {
     fn try_sign_digest(&self, msg_digest: D) -> Result<recoverable::Signature, Error> {
-        let z = <Scalar as Reduce<U256>>::from_be_bytes_reduced(msg_digest.finalize_fixed());
-        let k = rfc6979_generate_k::<_, D>(&self.inner, &z, &[]);
-        let (signature, recid) = self.inner.try_sign_prehashed(**k, z)?;
+        let digest = msg_digest.finalize_fixed();
+        let (signature, recid) = self.inner.try_sign_prehashed_rfc6979::<D>(digest, &[])?;
         let recoverable_id = recid.ok_or_else(Error::new)?.try_into()?;
         recoverable::Signature::new(&signature, recoverable_id)
     }
@@ -117,7 +139,16 @@ where
 
 impl<D> RandomizedDigestSigner<D, Signature> for SigningKey
 where
-    D: BlockInput + FixedOutput<OutputSize = U32> + Clone + Default + Reset + Update,
+    D: CoreProxy + Digest + FixedOutput<OutputSize = U32>,
+    D::Core: BlockSizeUser
+        + BufferKindUser<BufferKind = Eager>
+        + Clone
+        + Default
+        + FixedOutputCore
+        + HashMarker
+        + OutputSizeUser<OutputSize = D::OutputSize>,
+    <D::Core as BlockSizeUser>::BlockSize: IsLess<typenum::U256>,
+    Le<<D::Core as BlockSizeUser>::BlockSize, typenum::U256>: NonZero,
 {
     fn try_sign_digest_with_rng(
         &self,
@@ -131,19 +162,27 @@ where
 
 impl<D> RandomizedDigestSigner<D, recoverable::Signature> for SigningKey
 where
-    D: BlockInput + FixedOutput<OutputSize = U32> + Clone + Default + Reset + Update,
+    D: CoreProxy + Digest + FixedOutput<OutputSize = U32>,
+    D::Core: BlockSizeUser
+        + BufferKindUser<BufferKind = Eager>
+        + Clone
+        + Default
+        + FixedOutputCore
+        + HashMarker
+        + OutputSizeUser<OutputSize = D::OutputSize>,
+    <D::Core as BlockSizeUser>::BlockSize: IsLess<typenum::U256>,
+    Le<<D::Core as BlockSizeUser>::BlockSize, typenum::U256>: NonZero,
 {
     fn try_sign_digest_with_rng(
         &self,
         mut rng: impl CryptoRng + RngCore,
         msg_digest: D,
     ) -> Result<recoverable::Signature, Error> {
-        let mut added_entropy = FieldBytes::default();
-        rng.fill_bytes(&mut added_entropy);
+        let mut ad = FieldBytes::default();
+        rng.fill_bytes(&mut ad);
 
-        let z = <Scalar as Reduce<U256>>::from_be_bytes_reduced(msg_digest.finalize_fixed());
-        let k = rfc6979_generate_k::<_, D>(&self.inner, &z, &added_entropy);
-        let (signature, recid) = self.inner.try_sign_prehashed(**k, z)?;
+        let digest = msg_digest.finalize_fixed();
+        let (signature, recid) = self.inner.try_sign_prehashed_rfc6979::<D>(digest, &ad)?;
         let recoverable_id = recid.ok_or_else(Error::new)?.try_into()?;
         recoverable::Signature::new(&signature, recoverable_id)
     }
@@ -154,11 +193,12 @@ impl SignPrimitive<Secp256k1> for Scalar {
     fn try_sign_prehashed<K>(
         &self,
         ephemeral_scalar: K,
-        z: Scalar,
+        z: FieldBytes,
     ) -> Result<(Signature, Option<ecdsa_core::RecoveryId>), Error>
     where
-        K: Borrow<Scalar> + Invert<Output = Scalar>,
+        K: Borrow<Scalar> + Invert<Output = CtOption<Scalar>>,
     {
+        let z = <Self as Reduce<U256>>::from_be_bytes_reduced(z);
         let k_inverse = ephemeral_scalar.invert();
         let k = ephemeral_scalar.borrow();
 
@@ -200,8 +240,7 @@ impl ConstantTimeEq for SigningKey {
 
 impl Debug for SigningKey {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        // TODO(tarcieri): use `finish_non_exhaustive` when stable
-        f.debug_tuple("SigningKey").field(&"...").finish()
+        f.debug_struct("SigningKey").finish_non_exhaustive()
     }
 }
 
