@@ -2,7 +2,8 @@
 
 use super::{recoverable, Error, Signature};
 use crate::{
-    AffinePoint, CompressedPoint, EncodedPoint, ProjectivePoint, PublicKey, Scalar, Secp256k1,
+    AffinePoint, CompressedPoint, EncodedPoint, FieldBytes, ProjectivePoint, PublicKey, Scalar,
+    Secp256k1,
 };
 use ecdsa_core::{hazmat::VerifyPrimitive, signature};
 use elliptic_curve::{
@@ -12,7 +13,8 @@ use elliptic_curve::{
     sec1::ToEncodedPoint,
     IsHigh,
 };
-use signature::{digest::Digest, DigestVerifier};
+use signature::digest::{Digest, FixedOutput};
+use signature::DigestVerifier;
 
 #[cfg(feature = "sha256")]
 use signature::PrehashSignature;
@@ -25,7 +27,7 @@ use core::str::FromStr;
 
 #[cfg(all(feature = "pem", feature = "serde"))]
 #[cfg_attr(docsrs, doc(cfg(all(feature = "pem", feature = "serde"))))]
-use elliptic_curve::serde::{de, ser, Deserialize, Serialize};
+use serdect::serde::{de, ser, Deserialize, Serialize};
 
 /// ECDSA/secp256k1 verification key (i.e. public key)
 ///
@@ -73,13 +75,13 @@ where
     Self: DigestVerifier<S::Digest, S>,
 {
     fn verify(&self, msg: &[u8], signature: &S) -> Result<(), Error> {
-        self.verify_digest(S::Digest::new().chain(msg), signature)
+        self.verify_digest(S::Digest::new_with_prefix(msg), signature)
     }
 }
 
 impl<D> DigestVerifier<D, Signature> for VerifyingKey
 where
-    D: Digest<OutputSize = U32>,
+    D: Digest + FixedOutput<OutputSize = U32>,
 {
     fn verify_digest(&self, digest: D, signature: &Signature) -> Result<(), Error> {
         self.inner.verify_digest(digest, signature)
@@ -88,7 +90,7 @@ where
 
 impl<D> DigestVerifier<D, recoverable::Signature> for VerifyingKey
 where
-    D: Digest<OutputSize = U32>,
+    D: Digest + FixedOutput<OutputSize = U32>,
 {
     fn verify_digest(&self, digest: D, signature: &recoverable::Signature) -> Result<(), Error> {
         self.inner
@@ -97,16 +99,17 @@ where
 }
 
 impl VerifyPrimitive<Secp256k1> for AffinePoint {
-    fn verify_prehashed(&self, z: Scalar, signature: &Signature) -> Result<(), Error> {
+    fn verify_prehashed(&self, z: FieldBytes, signature: &Signature) -> Result<(), Error> {
         let r = signature.r();
         let s = signature.s();
+        let z = <Scalar as Reduce<U256>>::from_be_bytes_reduced(z);
 
         // Ensure signature is "low S" normalized ala BIP 0062
         if s.is_high().into() {
             return Err(Error::new());
         }
 
-        let s_inv = s.invert().unwrap();
+        let s_inv = *s.invert();
         let u1 = z * s_inv;
         let u2 = *r * s_inv;
 
@@ -153,12 +156,6 @@ impl From<&VerifyingKey> for PublicKey {
     }
 }
 
-impl From<&AffinePoint> for VerifyingKey {
-    fn from(affine_point: &AffinePoint) -> VerifyingKey {
-        VerifyingKey::from_encoded_point(&affine_point.to_encoded_point(false)).unwrap()
-    }
-}
-
 impl From<ecdsa_core::VerifyingKey<Secp256k1>> for VerifyingKey {
     fn from(verifying_key: ecdsa_core::VerifyingKey<Secp256k1>) -> VerifyingKey {
         VerifyingKey {
@@ -179,6 +176,26 @@ impl ToEncodedPoint<Secp256k1> for VerifyingKey {
     }
 }
 
+impl TryFrom<AffinePoint> for VerifyingKey {
+    type Error = Error;
+
+    fn try_from(affine_point: AffinePoint) -> Result<VerifyingKey, Error> {
+        let inner = PublicKey::try_from(affine_point)
+            .map_err(|_| Error::new())?
+            .into();
+
+        Ok(VerifyingKey { inner })
+    }
+}
+
+impl TryFrom<&AffinePoint> for VerifyingKey {
+    type Error = Error;
+
+    fn try_from(affine_point: &AffinePoint) -> Result<VerifyingKey, Error> {
+        VerifyingKey::try_from(*affine_point)
+    }
+}
+
 impl TryFrom<&EncodedPoint> for VerifyingKey {
     type Error = Error;
 
@@ -187,6 +204,34 @@ impl TryFrom<&EncodedPoint> for VerifyingKey {
     }
 }
 
+impl From<VerifyingKey> for ProjectivePoint {
+    fn from(verifying_key: VerifyingKey) -> ProjectivePoint {
+        PublicKey::from(verifying_key.inner).into()
+    }
+}
+
+impl From<&VerifyingKey> for ProjectivePoint {
+    fn from(verifying_key: &VerifyingKey) -> ProjectivePoint {
+        PublicKey::from(verifying_key.inner).into()
+    }
+}
+
+impl TryFrom<ProjectivePoint> for VerifyingKey {
+    type Error = Error;
+
+    fn try_from(point: ProjectivePoint) -> Result<VerifyingKey, Error> {
+        AffinePoint::from(point).try_into()
+    }
+}
+
+impl TryFrom<&ProjectivePoint> for VerifyingKey {
+    type Error = Error;
+
+    fn try_from(point: &ProjectivePoint) -> Result<VerifyingKey, Error> {
+        AffinePoint::from(point).try_into()
+    }
+}
+
 #[cfg(feature = "pkcs8")]
 #[cfg_attr(docsrs, doc(cfg(feature = "pkcs8")))]
 impl TryFrom<pkcs8::SubjectPublicKeyInfo<'_>> for VerifyingKey {
